package io.xjar;

import org.apache.maven.plugin.AbstractMojoExecutionException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Map;
import org.apache.maven.model.Build;
import io.xjar.filter.XAllEntryFilter;
import io.xjar.filter.XAnyEntryFilter;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.filter.XMixEntryFilter;
import org.apache.maven.plugin.logging.Log;
import org.apache.maven.plugin.MojoExecutionException;
import io.loadkit.Resource;
import io.loadkit.Loaders;
import io.xjar.boot.XBoot;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import io.xjar.jar.XJar;
import org.apache.maven.model.Plugin;
import java.io.IOException;
import java.io.File;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.project.MavenProject;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugin.AbstractMojo;

@Mojo(name = "build", defaultPhase = LifecyclePhase.PACKAGE)
public class XBuilder extends AbstractMojo
{
    @Parameter(defaultValue = "${project}", readonly = true, required = true)
    private MavenProject project;
    @Parameter(property = "xjar.algorithm", required = true, defaultValue = "AES/CBC/PKCS5Padding")
    private String algorithm;
    @Parameter(property = "xjar.keySize", required = true, defaultValue = "128")
    private int keySize;
    @Parameter(property = "xjar.ivSize", required = true, defaultValue = "128")
    private int ivSize;
    @Parameter(property = "xjar.password", required = true)
    private String password;
    @Parameter(property = "xjar.sourceDir", required = true, defaultValue = "${project.build.directory}")
    private File sourceDir;
    @Parameter(property = "xjar.sourceJar", required = true, defaultValue = "${project.build.finalName}.jar")
    private String sourceJar;
    @Parameter(property = "xjar.targetDir", required = true, defaultValue = "${project.build.directory}")
    private File targetDir;
    @Parameter(property = "xjar.targetJar", required = true, defaultValue = "${project.build.finalName}.xjar")
    private String targetJar;
    @Parameter(property = "xjar.includes")
    private String[] includes;
    @Parameter(property = "xjar.excludes")
    private String[] excludes;
    @Parameter(property = "xjar.deletes")
    private String[] deletes;
    
    public void execute() throws MojoExecutionException, MojoFailureException {
        final Log log = this.getLog();
        final String packaging = this.project.getPackaging();
        if (!"jar".equalsIgnoreCase(packaging)) {
            log.info((CharSequence)("Skip for packaging: " + packaging));
            return;
        }
        try {
            if (log.isDebugEnabled()) {
                log.debug((CharSequence)("Using algorithm: " + this.algorithm));
                log.debug((CharSequence)("Using key-size: " + this.keySize));
                log.debug((CharSequence)("Using iv-size: " + this.ivSize));
                log.debug((CharSequence)("Using password: " + this.password));
            }
            final File src = new File(this.sourceDir, this.sourceJar);
            final File dest = new File(this.targetDir, this.targetJar);
            final File folder = dest.getParentFile();
            if (!folder.exists() && !folder.mkdirs() && !folder.exists()) {
                throw new IOException("could not make directory: " + folder);
            }
            log.info((CharSequence)("Building xjar: " + dest + " for jar: " + src));
            XMixEntryFilter<JarArchiveEntry> filter;
            if (XArray.isEmpty(this.includes) && XArray.isEmpty(this.excludes)) {
                filter = null;
                log.info((CharSequence)"Including all resources");
            }
            else {
                filter = (XMixEntryFilter<JarArchiveEntry>)XKit.all();
                if (!XArray.isEmpty(this.includes)) {
                    final XAnyEntryFilter<JarArchiveEntry> including = XKit.any();
                    for (final String include : this.includes) {
                        including.mix((XEntryFilter)new XIncludeAntEntryFilter(include));
                        log.info((CharSequence)("Including " + include));
                    }
                    filter.mix((XEntryFilter)including);
                }
                if (!XArray.isEmpty(this.excludes)) {
                    final XAllEntryFilter<JarArchiveEntry> excluding = XKit.all();
                    for (final String exclude : this.excludes) {
                        excluding.mix((XEntryFilter)new XExcludeAntEntryFilter(exclude));
                        log.info((CharSequence)("Excluding " + exclude));
                    }
                    filter.mix((XEntryFilter)excluding);
                }
            }
            final Build build = this.project.getBuild();
            final Map<String, Plugin> plugins = build.getPluginsAsMap();
            final Plugin plugin = plugins.get("org.springframework.boot:spring-boot-maven-plugin");
            if (plugin == null) {
                XJar.encrypt(src, dest, XKit.key(this.algorithm, this.keySize, this.ivSize, this.password), (XEntryFilter)filter);
            }
            else {
                final Object configuration = plugin.getConfiguration();
                if (configuration instanceof Xpp3Dom) {
                    final Xpp3Dom dom = (Xpp3Dom)configuration;
                    Xpp3Dom child = dom.getChild("executable");
                    final String executable = (child != null) ? child.getValue() : null;
                    if ("true".equalsIgnoreCase(executable)) {
                        String msg = "Unsupported to build an xjar for an <executable>true</executable> spring boot JAR file, ";
                        msg += "maybe you should upgrade xjar-maven-plugin dependency if it have been supported in the later versions,";
                        msg += "if not, delete <executable>true</executable> or set executable as false for the configuration of spring-boot-maven-plugin.";
                        throw new MojoFailureException(msg);
                    }
                    child = dom.getChild("embeddedLaunchScript");
                    final String embeddedLaunchScript = (child != null) ? child.getValue() : null;
                    if (embeddedLaunchScript != null) {
                        String msg = "Unsupported to build an xjar for an <embeddedLaunchScript>...</embeddedLaunchScript> spring boot JAR file, ";
                        msg += "maybe you should upgrade xjar-maven-plugin dependency if it have been supported in the later versions,";
                        msg += "if not, delete <embeddedLaunchScript>...</embeddedLaunchScript> for the configuration of spring-boot-maven-plugin.";
                        throw new MojoFailureException(msg);
                    }
                }
                XBoot.encrypt(src, dest, XKit.key(this.algorithm, this.keySize, this.ivSize, this.password), (XEntryFilter)filter);
            }
            final File root = this.project.getFile().getParentFile();
            for (int i = 0; this.deletes != null && i < this.deletes.length; ++i) {
                String delete = this.deletes[i];
                File dir = root;
                while (delete.startsWith("../")) {
                    dir = ((dir.getParentFile() != null) ? dir.getParentFile() : dir);
                    delete = delete.substring("../".length());
                }
                log.info((CharSequence)("Deleting file(s) matching pattern: " + dir.getCanonicalPath().replace('\\', '/') + "/" + delete));
                final Enumeration<Resource> resources = Loaders.ant(Loaders.file(dir)).load(delete);
                while (resources.hasMoreElements()) {
                    final Resource resource = resources.nextElement();
                    final URL url = resource.getUrl();
                    final String path = url.getPath();
                    final File file = new File(path);
                    log.debug((CharSequence)("Deleting file: " + file.getCanonicalPath()));
                    if (!file.delete()) {
                        log.warn((CharSequence)("Could not delete file: " + file));
                    }
                }
            }
        }
        catch (final MojoExecutionException | MojoFailureException e) {
            throw e;
        }
        catch (final Exception e2) {
            throw new MojoExecutionException("could not build xjar", e2);
        }
    }
    
    public MavenProject getProject() {
        return this.project;
    }
    
    public void setProject(final MavenProject project) {
        this.project = project;
    }
    
    public String getAlgorithm() {
        return this.algorithm;
    }
    
    public void setAlgorithm(final String algorithm) {
        this.algorithm = algorithm;
    }
    
    public int getKeySize() {
        return this.keySize;
    }
    
    public void setKeySize(final int keySize) {
        this.keySize = keySize;
    }
    
    public int getIvSize() {
        return this.ivSize;
    }
    
    public void setIvSize(final int ivSize) {
        this.ivSize = ivSize;
    }
    
    public String getPassword() {
        return this.password;
    }
    
    public void setPassword(final String password) {
        this.password = password;
    }
    
    public File getSourceDir() {
        return this.sourceDir;
    }
    
    public void setSourceDir(final File sourceDir) {
        this.sourceDir = sourceDir;
    }
    
    public String getSourceJar() {
        return this.sourceJar;
    }
    
    public void setSourceJar(final String sourceJar) {
        this.sourceJar = sourceJar;
    }
    
    public File getTargetDir() {
        return this.targetDir;
    }
    
    public void setTargetDir(final File targetDir) {
        this.targetDir = targetDir;
    }
    
    public String getTargetJar() {
        return this.targetJar;
    }
    
    public void setTargetJar(final String targetJar) {
        this.targetJar = targetJar;
    }
    
    public String[] getIncludes() {
        return this.includes;
    }
    
    public void setIncludes(final String[] includes) {
        this.includes = includes;
    }
    
    public String[] getExcludes() {
        return this.excludes;
    }
    
    public void setExcludes(final String[] excludes) {
        this.excludes = excludes;
    }
    
    public String[] getDeletes() {
        return this.deletes;
    }
    
    public void setDeletes(final String[] deletes) {
        this.deletes = deletes;
    }
}
